	xdef	START
	
	section	program,code
	
*************************************************
**         declaration des constantes          **
*************************************************
	
numtest	equ	20		* nombre de tests du clavier
delai	equ	100
	
PGCR	equ	$10001		* configuration globale
PACR	equ	$1000D
PBCR	equ	$1000F
PADDR	equ	$10005		* masque de lecture du port A
PBDDR	equ	$10007
PCDDR	equ	$10009
PADR	equ	$10011		* adresse du port A
PBDR	equ	$10013
PCDR	equ	$10019
	
	
*************************************************
**                debut du code                **
*************************************************
	
START
	move.b	#$00,PGCR	* initialisation du PIT
	move.b  #$C0,PACR
	move.b	#$C0,PBCR
	
	move.b	#%00101011,PADDR	* masque du clavier
	move.b	#$FF,PBDDR	* toutes les cathodes sont en ecriture
	move.b	#$03,PCDDR	* autorise clock($01) et reset($02) sur le MAX
	
	move.b	#$03,PCDR	* effectue un reset
	move.b	#$00,PCDR
	
	move.l	#af,A0
	move.b	#%01101111,(A0)+	* initiallisation du buffer lignes
	move.b	#%10101111,(A0)+
	move.b	#%11001111,(A0)+
	move.b	#%11100111,(A0)+
	move.b	#%11101011,(A0)+
	move.b	#%11101101,(A0)+
	move.b	#%11101110,(A0)

***      debut de la boucle principale        ***
	
boucleprincipale
	bsr	litclavier	* lit le clavier
	
	move.w	(cl),D0
	move.l	#af,A0
	move.b	#ch0,A1

	tst.w	D0		* si aucune touche n'est enfonce,
	beq	praffich	*  ne change pas l'affichage
	
	* move.b	$#0F,D1	* pour eviter les boucles infinies
prclav
	btst	#$0,D0		* si on trouve le premier bit allume
	beq	prrecopaffinit	* on affiche le caractere selectionne
	ror.w	#$1,D0		* sinon on regarde le bit suivant
	add.l	#$7,A1		* fait pointer A1 sur le caractere suivant
	bra	prclav		* dbra	D0,prclav	* pour eviter bouclage
	
	
prrecopaffinit
	move.l	#$6,D2		* 7 lignes a recopier
prrecopaffsuiv
	move.b	(A1)+,(A0)+	* recopie le buffer
	dbra	D2,prrecopaffsuiv	* regarde la ligne suivante
	
praffich
	bsr	afficheecran	* affiche l'ecran
	
	bra	boucleprincipale	* recommence
	
	trap	#15		* fin du programme (jamais atteint)
	dc.w	0
	
***           fin boucle principale           ***
	
*** debut de la routine de lecture du clavier ***
	
litclavier			* lecture du clavier dans cl
	move.b	#%00101011,PADDR	* lignes en lecture
	move.w	D0,D1		* enregistre l'ancienne valeur du clavier
	move.b	#numtest,D3	* reinitialise le compteur de tests
memetouche
	move.w	#$0,D0		* RAZ D0
	
bclav				* test ligne par ligne :
	rol.w	#$4,D0		* decale D0
	cmpi.b	#$0,D2		* mets le bon bit a 0 dans l'entre clavier
	bne	lbla1
	move.b	#%00101010,PADR
lbla1	
	cmpi.b	#$1,D2
	bne	lbla2
	move.b	#%00101001,PADR
lbla2	
	cmpi.b	#$2,D2
	bne	lbla3
	move.b	#%00100011,PADR
lbla3	
	cmpi.b	#$3,D2
	bne	lbla4
	move.b	#%00001011,PADR
lbla4	
	move.b	PADR,D7		* recupere le retour clavier dans D7
	
lblb0	
	btst	#$2,D7		* teste les touches de la ligne en cours
	bne	lblb1		* si le bit n'est pas zero, ne modifie pas D0
	bset	#$1,D0
lblb1	
	btst	#$4,D7
	bne	lblb2
	bset	#$2,D0
lblb2	
	btst	#$6,D7
	bne	lblb3
	bset	#$3,D0
lblb3	
	btst	#$7,D7
	bne	lblb4
	bset	#$4,D0
lblb4	
	dbra	D2,bclav	* il reste encore des lignes a lire
	
	cmp.w	D0,D1
	bne	litclavier	* si les touches ont chang, recommence
	
	dbra	D3,memetouche	* sinon teste encore une fois

	move.w	D1,(cl)		* sauvegarde le resultat de la lecture clavier
	
	rts			* fin de la procedure
	
***  fin de la routine de lecture du clavier  ***
	
***      debut de la routine d'affichage      ***
	
afficheecran			* affichage du buffer d'ecran sur la matrice
	move.l	#$03,PCDR	* synchronisation du MAX (reset)
	move.l	#$00,PCDR
	
	move.l	#$06,D0		* initialisation du compteur de ligne courant
	
	move.l	#af,A0		* met l'adresse de debut du buffer dans A0
	
affichesuiv
	move.b	#$01,PCDR	* clock pour eteindre la ligne
	move.b	#$00,PCDR
	
	move.b	(A0)+,PBDR	* mise a jour des anodes
	
	move.b	#$01,PCDR	* clock pour allumer la ligne
	move.b	#$00,PCDR
	
	bsr	tempo		* temporisation
	
	dbra	D0,affichesuiv	* ligne suivante
	
	rts
	
*** fin de la routine d'affichage de l'ecran  ***
	
***   debut de la routine de temporisation    ***
	
tempo
	move.l	#delai,D7
	
bcltempo
	nop
	dbra.l	D7,bcltempo
	
	rts
	
***    fin de la routine de temporisation     ***
	
	
*************************************************
**          declaration des variables          **
*************************************************
	
af	ds.b	8		* buffer ecran virtuel
ar	ds.b	8		* buffer ecran reel (apres permutation)
	
cl	ds.w	1		* buffer de clavier

ch0	dc.b	%11101111	* image des chiffres a afficher (0)
	dc.b	%11100111
	dc.b	%10001001
	dc.b	%10101101
	dc.b	%10001001
	dc.b	%11100111
	dc.b	%11101111
	
ch1	dc.b	%11101111	* image des chiffres a afficher (1)
	dc.b	%11100111
	dc.b	%11000111
	dc.b	%10100111
	dc.b	%11100111
	dc.b	%11000011
	dc.b	%11101111

ch2	dc.b	%11101111	* image des chiffres a afficher (2)
	dc.b	%11000011
	dc.b	%10101101
	dc.b	%11101011
	dc.b	%11000111
	dc.b	%10000001
	dc.b	%11101111

ch3	dc.b	%11101111	* image des chiffres a afficher (3)
	dc.b	%11000011
	dc.b	%10101101
	dc.b	%11100011
	dc.b	%10101101
	dc.b	%11000011
	dc.b	%11101111
	
ch4	dc.b	%11101111	* image des chiffres a afficher (4)
	dc.b	%11101011
	dc.b	%11100111
	dc.b	%11001011
	dc.b	%10000001
	dc.b	%11101011
	dc.b	%11101111
	
ch5	dc.b	%11101111	* image des chiffres a afficher (5)
	dc.b	%10000001
	dc.b	%10101111
	dc.b	%10000011
	dc.b	%11101101
	dc.b	%10000011
	dc.b	%11101111

ch6	dc.b	%11101111	* image des chiffres a afficher (0)
	dc.b	%11100001
	dc.b	%11001111
	dc.b	%10100011
	dc.b	%10101101
	dc.b	%11000011
	dc.b	%11101111
	
ch7	dc.b	%11101111	* image des chiffres a afficher (7)
	dc.b	%10000001
	dc.b	%11100111
	dc.b	%11001111
	dc.b	%10101111
	dc.b	%10101111
	dc.b	%11101111
	
ch8	dc.b	%11101111	* image des chiffres a afficher (8)
	dc.b	%11000011
	dc.b	%10101101
	dc.b	%11000011
	dc.b	%10101101
	dc.b	%11000011
	dc.b	%11101111

ch9	dc.b	%11101111	* image des chiffres a afficher (9)
	dc.b	%11000011
	dc.b	%10101101
	dc.b	%11000001
	dc.b	%11101101
	dc.b	%11000011
	dc.b	%11101111
	
chA	dc.b	%11101111	* image des chiffres a afficher (A)
	dc.b	%11100111
	dc.b	%11001011
	dc.b	%11001011
	dc.b	%10000001
	dc.b	%10101101
	dc.b	%11101111
	
chB	dc.b	%11101111	* image des chiffres a afficher (B)
	dc.b	%10000011
	dc.b	%10101101
	dc.b	%10000011
	dc.b	%10101101
	dc.b	%10000011
	dc.b	%11101111
	
chC	dc.b	%11101111	* image des chiffres a afficher (C)
	dc.b	%11000011
	dc.b	%10101101
	dc.b	%10101111
	dc.b	%10101101
	dc.b	%11000011
	dc.b	%11101111
	
chD	dc.b	%11101111	* image des chiffres a afficher (D)
	dc.b	%10000011
	dc.b	%10101101
	dc.b	%10101101
	dc.b	%10101101
	dc.b	%10000011
	dc.b	%11101111
	
chE	dc.b	%11101111	* image des chiffres a afficher (E)
	dc.b	%10000001
	dc.b	%10101111
	dc.b	%10000011
	dc.b	%10101111
	dc.b	%10000001
	dc.b	%11101111
	
	
chF	dc.b	%11101111	* image des chiffres a afficher (F)
	dc.b	%10000001
	dc.b	%10101111
	dc.b	%10000011
	dc.b	%10101111
	dc.b	%10101111
	dc.b	%11101111
	
	END
