GUILLOT NAHRY Gr4.2

/* definition des masques pour l'interpretation du clavier */
#define j_msk 0x7F
#define reset_msk 0x80
#define stop_msk 0x8000

/* definition des bits pour ecriture dans l'ordre des cathodes */
#define b0 0x08
#define b1 0x40
#define b2 0x02
#define b3 0x80
#define b4 0x20
#define b5 0x04
#define b6 0x01

/* adresse des buffers utiliss par les interrupts */
unsigned char * ptab_chute;
unsigned char * ptab_oqp;
unsigned char * ptab_j;

/* garde en memoire la derniere touche lue */
unsigned long lastclav = 0;

/* tables de convertion ordre cathodique/logique */
unsigned long permut[8]   = {3,6,1,7,5,2,0,4};
unsigned long depermut[8] = {6,2,5,0,7,5,1,3};

/* les differentes victoires possible */
enum Tvictoire {pasvictoire, joueur1, joueur2, matchnul, reset, stop} victoire;

/* variables asm pour 'testvictoire ()' */
extern unsigned long nbtestsvict;
extern unsigned long tabvictX, tabvictY;

/* fonctions exportes de l'assembleur */
extern void PITinit (unsigned long, char **, char **, char **);
extern unsigned long lectclavier (void);
extern void cli (void);
extern void sti (void);
extern void exxit (void);

/* affichage d'un caractere sur la console */
void output (char c) {
	char * uart;
 	uart = 32771;
	while ((int)(*uart & 4) == 0);
	uart = 32775;
	*uart = c;
}

/* affichage d'un chiffre hexa sur la console (pour debuggage) */
void outputhex (unsigned long i) {
	if (i < 10)
		output ((char)(i + '0'));
	else
		output ((char)(i + 'A' - 10));
}

/* affichage d'une chaine sur la console (avec retour charriot) */
void outputstring (char * str) {
	char c;
	while ((c = *(str++)) != 0)
		output (c);
	output ('\n');
}

/* fonction retournant la valeur du clavier si elle a change depuis la derniere lecture */
unsigned long lectclavieronce () {
	unsigned long clav;
	clav = lectclavier ();
	if (clav == lastclav)
		return 0;
	lastclav = clav;
	return clav;
}

/* efface tous les buffers d'affichage */
void cls (void) {
	unsigned long i;
	for (i = 0; i < 7; i++) {
		ptab_chute[6 - i] = (unsigned char) 0x00;
		ptab_oqp[6 - i] = (unsigned char) 0x00;
		ptab_j[6 - i] = (unsigned char) 0x00;
	}
}

/* gestion bit par bit des buffers aux coordonnees specifies, dans l'ordre des cathodes */
/* met le bit a 1 */
void setbit (char * ptab, unsigned long x, unsigned long y) {
	if ((x < 7) && (y < 7))
		ptab[y] |= (1 << permut[x]);
}

/* met le bit a 0 */
void clrbit (char * ptab, unsigned long x, unsigned long y) {
	if ((x < 7) && (y < 7))
		ptab[y] &= ~(1 << permut[x]);
}

/* renvoie la valeur du bit, et 0 si hors limites */
unsigned long getbit (char * ptab, unsigned long x, unsigned long y) {
	if ((x > 6) || (y > 6))
		return 0;
	return 1 & (ptab[y] >> permut [x])
}	

/* renvoie 0 si la premiere case de la colonne est libre, apres y avoir mis le jeton, renvoie 1 sinon */
unsigned long joueurjoue (unsigned long joueur, unsigned long colonne) {
/* si la case est dja occupe, renvoie 1 */
	if (getbit (ptab_oqp, colonne, 6) || getbit (ptab_chute, colonne, 6))
		return 1;

/* sinon joue et renvoie 0 */
	if (joueur == 2)
		setbit (ptab_j, colonne, 6);
/* on positionne le bit de joueur avant le bit de chute pour ne pas produire d'erreur si une interruption a lieu ici */
	setbit (ptab_chute, colonne, 6);
	return 0;
}

/* fonction qui renvoie 0 si le jeu doit continuer, ou 1 apres avoir defini 'victoire' */
unsigned long testvictoire () {
	unsigned long total;
	unsigned long jbit;
	unsigned long i;
	unsigned long x, y;
	unsigned long * tabX;
	unsigned long * tabY;

/* on charge l'adresse des tableaux definis en assembleur */
	tabX = &tabvictX;
	tabY = &tabvictY;

/* on interdit les interrupts afin d'viter les erreurs en cas de modification des donnees pendant la phase critique */
	cli ();
	if (nbtestsvict == 0) {
		sti ();
		return 0;
	}
	x = tabX[nbtestsvict];
	y = tabY[nbtestsvict];
	nbtestsvict--;
	sti ();
/* fin de la phase critique */

	x = depermut[x];	/* traduction de x qui est dans l'ordre des cathodes */

/* informations de debuggage */
/*	outputhex (x);
	output (',');
	outputhex (y);
	output ('\n');*/

/* on regarde a quel joueur appartient le jeton */
	jbit = getbit (ptab_j, x, y);

/* test horizontal */
	total = 1;
/* vers la gauche */
	for (i = 1; i < 4; i++) {
		if (!getbit (ptab_oqp, x - i, y) || (getbit (ptab_j, x - i, y) != jbit))
			break;
		total++;
	}
/* et vers la droite */
	for (i = 1; i < 4; i++) {
		if (!getbit (ptab_oqp, x + i, y) || (getbit (ptab_j, x + i, y) != jbit))
			break;
		total++;
	}
/* on regarde le nombre total de jetons aligns */
	if (total > 3) {
/* victoire ! */
		if (!jbit)
			victoire = joueur1;
		else
			victoire = joueur2;
		return 1;
	}

/* test vertical */
	total = 1;
/* vers le bas */
	for (i = 1; i < 4; i++) {
		if (!getbit (ptab_oqp, x, y - i) || (getbit (ptab_j, x, y - i) != jbit))
			break;
		total++;
	}
/* vers le haut */
	for (i = 1; i < 4; i++) {
		if (!getbit (ptab_oqp, x, y + i) || (getbit (ptab_j, x, y + i) != jbit))
			break;
		total++;
	}
	if (total > 3) {
/* victoire ! */
		if (!jbit)
			victoire = joueur1;
		else
			victoire = joueur2;
		return 1;
	}

/* test sur la diagonale */
	total = 1;
/* bas-gauche */
	for (i = 1; i < 4; i++) {
		if (!getbit (ptab_oqp, x - i, y - i) || (getbit (ptab_j, x - i, y - i) != jbit))
			break;
		total++;
	}
/* haut-droite */
	for (i = 1; i < 4; i++) {
		if (!getbit (ptab_oqp, x + i, y + i) || (getbit (ptab_j, x + i, y + i) != jbit))
			break;
		total++;
	}
	if (total > 3) {
		if (!jbit)
			victoire = joueur1;
		else
			victoire = joueur2;
		return 1;
	}

/* test sur l'antidiagonale */
	total = 1;
/* haut-gauche */
	for (i = 1; i < 4; i++) {
		if ((getbit (ptab_oqp, x - i, y + i) == 0) || (getbit (ptab_j, x - i, y + i) != jbit))
			break;
		total++;
	}
/* bas-droite */
	for (i = 1; i < 4; i++) {
		if ((getbit (ptab_oqp, x + i, y - i) == 0) || (getbit (ptab_j, x + i, y - i) != jbit))
			break;
		total++;
	}
	if (total > 3) {
		if (!jbit)
			victoire = joueur1;
		else
			victoire = joueur2;
		return 1;
	}

/* si le jeton est sur la premiere ligne, on regarde si le tableau est plein */
	if (y == 6) {
		total = 0;
		for (i = 0; i < 7; i++) {
			if (getbit (ptab_oqp, i, 6))
				total++;
		}
		if (total > 6) {
/* match nul ! */
			victoire = matchnul;
			return 1;
		}
	}

	return 0;
}

/* fonction principale */
void jeu (void) {
	unsigned long clav;
	unsigned long joueur;
	unsigned long colonne;

/* on reinitialise les buffers et les variables du jeu */
	cls ();
	nbtestsvict = 0;
	victoire = pasvictoire;
	joueur = 1;

/* boucle tant que le jeu n'est pas fini */
	while (victoire == pasvictoire) {

/* appel de 'testvictoire ()' */
		if (testvictoire ())
			break;

		clav = lectclavieronce ();
/* on attend qu'une touche soit presse */
		if (clav == 0)
			continue;

/* gestion de stop/reset */
		if (clav & reset_msk) {
			victoire = reset;
			break;
		}
		if (clav & stop_msk) {
			victoire = stop;
			break;
		}

/* on masque le clavier pour n'avoir que les touches correspondant au joueur en cours */
		if (joueur == 1)
			clav &= j_msk;
		if (joueur == 2)
			clav = (clav >> 8) & j_msk;

/* pour chaque touche presse par le joueur, on regarde si on peut jouer dans la colonne correspondant */
		for (colonne = 0; colonne < 7; colonne++)
			if (clav & (1 << colonne))
				if (joueurjoue (joueur, colonne) == 0) {
/* et si c'est valable, on change de joueur */
					if (joueur == 1)
						joueur = 2;
					else
						joueur = 1;
					break;
				}
	}
}

/* affiche un cran signifiant la victoire du joueur 1 */
void player1wins (void) {
	outputstring ("player 1 wins !");
	ptab_oqp[0] = b2+b3+b4+b5;
	ptab_oqp[1] = b3+b4;
	ptab_oqp[2] = b3+b4;
	ptab_oqp[3] = b1+b2+b3+b4;
	ptab_oqp[4] = b2+b3+b4;
	ptab_oqp[5] = b3+b4;
	ptab_oqp[6] = b4;
}

/* affiche un cran signifiant la victoire du joueur 2 */
void player2wins (void) {
	outputstring ("player 2 wins !");
	ptab_oqp[0] = b1+b2+b3+b4+b5;
	ptab_oqp[1] = b2+b3+b4+b5;
	ptab_oqp[2] = b3+b4;
	ptab_oqp[3] = b4+b5;
	ptab_oqp[4] = b1+b4+b5;
	ptab_oqp[5] = b1+b2+b4+b5;
	ptab_oqp[6] = b2+b3+b4;
}

void main (void) {
	unsigned long clav;
	unsigned long i;

/* setup du PIT et initialisation des variables de communication avec le code en assembleur */
       	PITinit (400, &ptab_chute, &ptab_oqp, &ptab_j);

	while (1) {
		victoire = reset;
/* on joue tant que le jeu a cause d'un reset */
		while (victoire == reset)
			jeu ();

/* on efface tout */
		cls ();
		nbtestsvict = 0;
	
		switch (victoire) {
		case matchnul:
			outputstring ("match nul !");
			break;
		case joueur1:
			player1wins ();
			break;
		case joueur2:
			player2wins ();
			break;
		case stop:
			outputstring ("STOP !!!");
/* on termine le jeu */
   	        	exxit ();
			break;
		case pasvictoire:
			outputstring ("mais pourquoi on s'arrete ?");
		}

/* on affiche en permanence jusqu'a pression d'une touche au clavier */
		while ((clav = lectclavieronce ()) == 0)
			if (i++ > 0xFFF)
/* de temps en temps on alterne entre affichage clignotant et fixe */
				for (i = 0; i < 7; i++)
					ptab_j[i] ^= 0xEF;

/* et on recommence */
	}
}

