* 68000 Startup Routine for Crossware C Compiler
* This startup routine is provided as an example only.  It must be modified to suit your
* target system.

* The following block will be maintained by the Embedded Development Studio
* DO NOT EDIT
*{{EDS_STARTUP(C68000.ASM)
__NoRom equ  1
__LowestRomLocation equ  0400400H
__HighestRomLocation equ  047FFFFH
__LowestRamLocation equ  0479000H
__HighestRamLocation equ  047FFFFH
*}}EDS_STARTUP
* END OF DO NOT EDIT
* This is the end of the block that will be maintained by the Embedded Development Studio

	xref	_main,__initdata,__HP
	xdef	__cstart,_exit

******************************************************************************************

* These sections are required by the Crossware C Compiler:

	section	__STACK,data,high	Linker places this at highest available ram location
	ds.w	1			1 word minimum to create section

	section __HEAP,data,postpone	Linker places this after all ram sections except __STACK
	ds.w	1			1 word minimum to create section


******************************************************************************************

* Modify the code below to suit your particular target system and program requirements.


**** Start of startup code for separate code and data ****

**** This code will be included when __NoRom is zero. Ie when the
**** "Code and data in ram" option in the linker settings has not been checked

* We are assuming here that the code will go in rom.
* The address of the __STACK section is placed at address zero 
* and so becomes the supervisor stack as well as the working stack
* for the C compiler.  This means that the chip will remain in supervisor
* at all times.
* Execution immediately jumps to __cstart.
* The compiler generates a reference to '__cstart' to force the linker to include at least
* one module with this label. Therefore you should use '__cstart' in this file otherwise
* you will get an error message from the linker or a second startup module (extracted
* from the library) will be included in the final program.
	
	ifeq	__NoRom

	org     0
	dc.l    __STACK		Initialise supervisor stack for C compiler
	dc.l    __cstart	Jump to __cstart on power up
        
	SECTION __CODE
       
__cstart
	move.l	#__HEAP,__HP	Initialise heap pointer. (You must do this if you are using the heap.  Note printf() etc use the heap.)
	jsr	__initdata	__initdata initialises all C variables and must always called
	jsr	_main
_exit

	endc				
**** End of startup code for separate code and data ****

**** Start of startup code for combined code and data ****

**** This code will be included when __NoRom is non-zero. Ie when the
**** "Code and data in ram" option in the linker settings has been checked
* We are assuming here that your target board already has a monitor or debugger
* running on it and that you are loading your program and data into data space
* on that target board.
* We are also assuming that this monitor/debugger is running in supervisor mode
* with its own stack. We need to set up the C compiler stack as the user stack
* and we may need to switch to user mode.
* Before switching to user mode by clearing the S bit in the status register, 
* we need to ensure that we are not already in user mode - because MOVE <ea>,SR is
* a privileged instruction.  This is easy on the M68000 - just load the status
* register and check the 'S'bit.

	ifne	__NoRom

	org	__LowestRamLocation	Lowest user ram location
	even				Just in case an odd address has be put into the linker settings
	xdef __START
__START					* tell the linker the start address
__cstart
	move	sr,d0		Get status reg
*	andi	#$2000,d0	Isolate supervisor bit
*	beq.s	skip1		Branch if the chip is already in user mode
	ori	#$2000,d0
	move	d0,sr
	bra.s	skip1

* The chip is in supervisor mode if here
	move	sr,d0		Get status register
	andi	#$dfff,d0	Clear S bit in status register
	move	d0,sr		Set user mode
skip1	movea.l	#__STACK,a7	Set user stack pointer.  This is essential.  The C compiler must have a stack.
	move.l	#__HEAP,__HP	Initialise heap pointer. (You must do this if you are using the heap.  Note printf() etc use the heap.)
	jsr	__initdata	__initdata initialises all C variables and must always called
	jsr	_main
_exit

* Where execution goes from here depends on your target system. 
* A simple rts might be sufficient to return control to your target systems monitor/debugger.
* The code below is for a particular target board that requires a trap 11.

	trap	#15		Call monitor RENTER command
	dc.w	0		Re-enter monitor
	
	endc
**** End of startup code for combined code and data ****
	
